package DownImgs;

#
#  todo: proxy, retry
#

use strict;
use warnings;

use File::Path;
use LWP::UserAgent;

# for geocoding
use URI::Escape;

sub new {
  my $pkg = shift;
  my $this = {
	      ua => LWP::UserAgent->new,
	      timeout => 10,
	      retry => 3,
	      verbose => 0,
	      base_dir => 'imgs',
	      index_max => 4,
	      index => 0,
	      agent => "Mozilla/5.0 (Windows; U; Windows NT 5.1; ja; rv:1.8.1.18) Gecko/20081029 Firefox/2.0.0.18",
	      top_url => 'http://maps.google.co.jp/',
	      output => 'file',  # 'file' or 'return'
	      @_
	     };

  $this->{ua}->agent( $this->{agent} );
  $this->{ua}->timeout( $this->{timeout} );
  bless $this, ref $pkg || $pkg;
}


sub fetchver {
  my ( $this ) = @_;

  my $req = new HTTP::Request GET => $this->{top_url};

  # implement retry?
  my $res = $this->{ua}->request( $req );
  unless ( $res->is_success ) {
    $this->{status_line} = $res->status_line;
    if ( $this->{verbose} ) {
      print "failed to obtain GMver (1)\n";
    }
    return undef;
  }

  $_ = $res->content;
  if ( m#http://mt\d\.google\.com/vt/lyrs=m\@(\d+)# ) {
    if ( $this->{verbose} ) {
      print "obtained GMver $1\n";
    }
    $this->{gmver} = $1;
    return $1;
  }
  if ( $this->{verbose} ) {
    print "failed to obtain GMver (2)\n";
  }
  undef;
}

sub getver {
  my ( $this ) = @_;

  $this->{gmver};
}

sub setver {
  my ( $this, $ver ) = @_;

  $this->{gmver} = $ver;
}

#
# 地図を１枚ダウンロード
#
sub down_map1
{
  my ( $this, $zoom, $x, $y, $index ) = @_;

  unless ( defined $index ) {
    $index = $this->{index}++;
    $this->{index} = 0 if $this->{index} >= $this->{index_max};
  }

  my $zstr = sprintf("%02d", $zoom);
  my @xstr = split( //, sprintf("%010d", $x) );
  my @ystr = split( //, sprintf("%010d", $y) );

  my $imgpath = "$this->{base_dir}/$zstr";
  foreach my $i ( 0 .. 9 ) {
    $imgpath .= "/" . shift( @xstr ) . shift( @ystr );
  }
  $imgpath .= ".png";

  my $imgdir = $imgpath;
  $imgdir =~ s/\/[^\/]*$//;
  if (! -d $imgdir) {
    mkpath($imgdir);
  }

  if (-e $imgpath) {
    my $imgsize = (stat($imgpath))[7];
    if ($imgsize == 0) {
      unlink($imgpath);
    }
  }

  if ( $this->{verbose} ) {
    printf "Downloading z = %02d,  x = %010d, y = %010d from mt%d ...(dest:$imgpath) gmver=%d\n", $zoom, $x, $y, $index, $this->{gmver};
  }
  my $target_url = sprintf("http://mt%d.google.com/vt/lyrs=m\@%s&hl=ja&x=$x&s=&y=$y&z=%d", 
			   $index, $this->{gmver}, (17 - ($zoom-2)));

  my $req = new HTTP::Request GET => $target_url;
  $req->referer( "http://maps.google.com/" );

  # implement retry?
  my $res = $this->{ua}->request( $req );
  unless ( $res->is_success ) {
    if ( $this->{verbose} ) {
      print "status: ", $res->status_line, "\n";
    }
    $this->{status_line} = $res->status_line;
    return undef;
  }

  if ( $this->{output} eq 'file' ) {
    open( W, "> $imgpath" ) or die;
    binmode( W );
    print W $res->content;
    close( W );

    return 1;
  } elsif ( $this->{output} eq 'return' ) {
    return $res->content;
  }
}

sub geocoding {
  my ( $this, $addr ) = @_;

  my $addr_uri = uri_escape_utf8( $addr );
  my $url = "http://www.geocoding.jp/api/?q=$addr_uri&v=1.1";
  my $req = new HTTP::Request GET => $url;
  my $res = $this->{ua}->request( $req );

  unless ( $res->is_success ) {
    if ( $this->{verbose} ) {
      print "status: ", $res->status_line, "\n";
    }
    $this->{status_line} = $res->status_line;
    return undef;
  }

  my $xml = $res->content;

  my ( $lat ) = $xml =~ /<lat>(.+)<\/lat>/;
  my ( $lng ) = $xml =~ /<lng>(.+)<\/lng>/;

  return unless defined $lat && defined $lng;
  "$lng,$lat";
}


1;
