#!/usr/bin/perl

use strict;
use warnings;
use GD;

my $linewidth = 1;
my $outdir = "pgmap_data";

my %color = qw/
  black 000000
  white ffffff
  red ff0000
  green 00ff00
  blue 0000ff
  gray bebebe
  navy 000080
  skyblue 87ceeb
  steelblue 4682b4
  cyan 00ffff
  darkgreen 006400
  palegreen 98fb98
  lawngreen 7cfc00
  forestgreen 228b22
  yellow ffff00
  gold ffd700
  beige f5f5dc
  brown a52a2a
  salmon fa8072
  orange ffa500
  tomato ff6347
  pink ffc0cb
  maroon b03060
  magenda ff00ff
  violet ee82ee
  plum dda0dd
/;

my $poly = GD::Polygon->new;
my @pt = qw/ 1 1   14 1   14 10   12 10   7 15   9 10  1 10 /;
while ( @pt ) {
  $poly->addPt( shift @pt, shift @pt );
}

foreach my $color ( keys %color ) {
  my $cc = $color{$color};
  my @rgb;
  foreach my $k ( 0 .. 2 ) {
    push( @rgb, hex( substr( $cc, $k * 2, 2 ) ) );
  }
#  print "rgb = @rgb ($cc1,$cc2,$cc3)\n";
  $color{$color} = {code=>$color{$color}, array=>[@rgb]};
}

foreach my $color ( keys %color ) {
#  print "making $color($color{$color}->{array}:@{$color{$color}->{array}})\n";
  my $im = GD::Image->new( 16, 16 );
  my $white = $im->colorAllocate( @{$color{white}->{array}} );
  my $black = $im->colorAllocate( @{$color{black}->{array}} );
  my $main = $im->colorAllocate( @{$color{$color}->{array}} );

  $im->transparent( $white );
  $im->filledPolygon( $poly, $main );
  $im->setThickness( $linewidth );
  $im->polygon( $poly, $black );

  open my $out, '>', "$outdir/icon-$color.png" or die $!;
  binmode $out;
  print $out $im->png;
  close $out;
}




#my @pt21 = qw/ 0 -7   -5 7  0 2  5 7 /;
#my @pt21 = qw/ -7 0  7 5  2 0  7 -5 /;
my @pt21 = qw/ 7 0  -7 -5  -2 0  -7 5 /;


my $angle_num = 24;
foreach my $angle_index ( 0 .. $angle_num - 1 ) {
  my $rad = $angle_index * 2 * 3.14159 / $angle_num;
  my $im = GD::Image->new( 14, 14 );

  my $white = $im->colorAllocate( @{$color{white}->{array}} );
  my $black = $im->colorAllocate( @{$color{black}->{array}} );
  my $main = $im->colorAllocate( @{$color{red}->{array}} );

  my $poly = GD::Polygon->new;
  my @pt22 = @pt21;
  while ( @pt22 ) {
    my ( $x, $y ) = ( shift @pt22, shift @pt22 );
    ( $x, $y ) = ( $x * cos( $rad ) - $y * sin( $rad ), $x * sin( $rad ) + $y * cos( $rad ) );
    $poly->addPt( $x + 7, $y + 7 );
  }

  $im->transparent( $white );
  $im->filledPolygon( $poly, $main );

  open my $out, '>', "$outdir/icon-$angle_index.png" or die $!;
  binmode $out;
  print $out $im->png;
  close $out;
}

